;--------------------------------------------------------------
; SOSJ Music Loader V1.0 (aka Sausage) for Dragon/Coco
;
; Music player by Simon Jonassen
; Simon's player source available here: http://www.roust-it.dk/coco/dragon/
;
; Loader by Stewart Orchard
;
; Plays a 2 voice track converted from xm format
; Conversion script provided by UTZ http://www.irrlichtproject.de/
;
; 1 DAC update and 1 tape sample per 98 cycle loop = approx 9.1KHz
;
; Loads and runs standard format tape binaries. Just need to be careful
; that loaded file doesn't overwrite loader.
; Performs usual I/O error check by evaluating block checksums
;
; Input phase is assumed from ROM phase flag in $84
; therefore requires load target file to be the same phase
; as the file that gets this loader into memory
;
; Assembled using asm6809: http://www.6809.org.uk/asm6809/
;
; Required source files:
; load9k.asm		(this file)
; macros.asm		(for 2-24 cycle delay macro)
; music.asm			(music data)
; message.asm		(scrolling message content in 6847 codes)
; textscreen.asm	(512 byte text loading screen)
;
; Other files:
; xm2cocopp.pl		(xm to assembler conversion script by UTZ)
; message.txt		(scrolling message content in ascii)
; make_msg.vbs		(script to convert ascii to 6847)
; music.xm			(xm music file as saved by MilkyTracker)

; Notes:
; xm2cocopp.pl seems to prefer MilkyTracker xm files
; first pattern in song should be number 0
; fdb 0 written at end of pattern list should be manually removed
; Turn off oversampling & interpolation in tracker to get a better
; idea of final sound.
;
;--------------------------------------------------------------

		include "macros.asm"


pia1	equ $ff20

phase	equ $84		; phase flag updated by rom routines
MIDVAL	equ 6		; 0/1 discrimination threshold (5-7)

FILE_HEADER	equ $1c00	; load address of filename block

MSG_WINSTART	equ $5e0	; start of scroll window
MSG_WINLEN		equ $20		; size of scroll window

		org $600

start
		pshs cc,dp

		orcc #$50
		
		lda #(start >> 8)
		tfr a,dp
		setdp (start >> 8)

		jsr setup

		sts tempstk+2	; save stack

		; first task for loader is acquire sync byte $3c
		lds #sync_st

		; first task for sequencer is check for end of pattern
		ldy #seq_st

; *** START OF MAIN LOOP ***

loop
		; service note (37 cycles)
		
sum1	ldd #$0000		; [3]
freq1	addd #$0000 	; [4]
		std <sum1+1		; [5]
sum2	ldd #$0000 		; [3]
freq2	addd #$0000 	; [4]
		std <sum2+1		; [5]
		lsra			; [2]
		adda <sum1+1	; [4]
		rora			; [2]
		sta $ff20		; [5]

		; service loader & sequencer (58 cycles)
		; s points to next loader routine
		; y points to next sequencer routine
		;
		; loader routine should end with jmp ,y to service sequencer
		;   or jmp loop to use sequencer time slot.
		; loader uses 33 cycles (or 58 if taking sequencer slot)
		;
		; sequencer routine should end with jmp loop.
		; sequencer uses 25 cycles
		
		jmp ,s			; [3]

; *** END OF MAIN LOOP ***


;---------------------------------------

ldaddr	rmb 2	; load start address
execad	rmb 2	; exec start address
timer	rmb 1	; wave timer
bitcnt	rmb 1	; bit count for loader
bytcnt	rmb 1	; byte count for loader
hdflag	rmb 1	; header/block flag
blktyp	rmb 1	; block type (00 = filename, 01 = data, ff = eof)
blklen	rmb 1	; block length
cksum	rmb 1	; calculated block cksum
blnkflg	rmb 1	; loader blink flag

curnote	rmb 2	; pointer to current note
curpat	rmb 2	; pointer to current pattern
ntcnt	rmb 2	; note duration counter

msgstart	rmb 2	;
msgptr		rmb 2	;
scrnptr		rmb 2	;

;--------------------------------------------------------------
; NOTE SEQUENCER CODE
;
; All routines padded to 25 cycles
;--------------------------------------------------------------

; sequencer starts here
;
; check if end of pattern	
seq_st
		mac_delay_cycles 6
		
		lda [curnote]	; [9]
		bpl 1f			; [3] -ve note number marks end of pattern

		ldy #seq1		; [4] go to next pattern
		jmp loop		; [3] *19*
		
1		ldy #seq_pl		; [4] go to note setup
		jmp loop		; [3] *19*

; get next pattern
seq1
		mac_delay_cycles 3
		
		ldx curpat		; [5]
		leax 2,x		; [5]
		stx curpat		; [5]
		
		ldy #seq2		; [4]
		jmp loop		; [3] *22*

; check if end of patterns
seq2
		mac_delay_cycles 6
		
		ldx curpat		; [5]
		cmpx #PAT0		; [4] end of pattern list?
		blo	1f			; [3]

		ldy #seq3		; [4] reset pattern pointer
		jmp loop		; [3] *19*

1		ldy #seq4		; [4] get first note of pattern
		jmp loop		; [3] *19*

; reset pattern pointer
seq3
		mac_delay_cycles 10
		
		ldx #PATTERNS	; [3]
		stx curpat		; [5]

		ldy #seq4		; [4] get first note of pattern
		jmp loop		; [3] *15*

; set note pointer to start of pattern
seq4
		mac_delay_cycles 3
		
		ldx [curpat]	; [10]
		stx curnote		; [5]

		ldy #seq_pl		; [4] note setup
		jmp loop		; [3] *22*
		
		
; start setting up voices
seq_pl
		ldd [curnote]	; [10]
		sta note1+1		; [4]
		stb note2+1		; [4]
		
		ldy #seq_pl2	; [4]
		jmp loop		; [3] *25*

; set up voice 1 frequency		
seq_pl2		
note1	lda #0			; [2]
		lsla			; [2]
		ldx #freqtab	; [3]
		ldx a,x			; [6]
		stx freq1+1		; [5]

		ldy #seq_pl3	; [4]
		jmp loop		; [3] *25*

; set up voice 2 frequency		
seq_pl3
note2	lda #0			; [2]
		lsla			; [2]
		ldx #freqtab	; [3]
		ldx a,x			; [6]
		stx freq2+1		; [5]

		ldy #seq_pl4	; [4]
		jmp loop		; [3] *25*

; increment note pointer
seq_pl4
		mac_delay_cycles 3
		
		ldx curnote		; [5]
		leax 2,x		; [5]
		stx curnote		; [5]

		ldy #scroll_1	; [4]
		jmp loop		; [3] *22*

		
; reset pointers to start of scroll window
scroll_1
		ldx msgstart	; [5]
		stx msgptr		; [5]
		ldx #MSG_WINSTART	; [3]
		stx scrnptr		; [5]
		ldy #scroll_2	; [4]
		jmp loop		; [3] *25*
		
; copy a character from message to screen
scroll_2
		lda [msgptr]	; [9]
		sta [scrnptr]	; [9]
		ldy #scroll_3	; [4]
		jmp loop		; [3] *25*

; advance message pointer
scroll_3
		mac_delay_cycles 3
		ldx msgptr		; [5]
		leax 1,x		; [5]
		stx msgptr		; [5]
		ldy #scroll_4	; [4]
		jmp loop		; [3] *22*

; advance screen pointer	
scroll_4
		mac_delay_cycles 3
		ldx scrnptr		; [5]
		leax 1,x		; [5]
		stx scrnptr		; [5]
		ldy #scroll_5	; [4]
		jmp loop		; [3] *22*

; check if end of window
; keep looping until window is filled
scroll_5
		mac_delay_cycles 6
		ldx scrnptr		; [5]
		cmpx #MSG_WINSTART+MSG_WINLEN	; [4]
		bne	1f			; [3]
		ldy #scroll_6	; [4]
		jmp loop		; [3] *19*
1		ldy #scroll_2	; [4]
		jmp loop		; [3] *19*

; advance start of message
; this makes the message scroll
scroll_6
		mac_delay_cycles 3
		ldx msgstart	; [5]
scr_mod	leax 2,x		; [5]
		stx msgstart	; [5]
		ldy #scroll_6b	; [4]
		jmp loop		; [3] *22*

scroll_6b
		mac_delay_cycles 6
		lda scr_mod+1	; [5]
		eora #2			; [2]
		sta scr_mod+1	; [5]
		ldy #scroll_7	; [4]
		jmp loop		; [3] *19*

; check if scrolled to end of message
scroll_7
		mac_delay_cycles 6
		ldx msgstart	; [5]
		cmpx #MSG_TEXT_END-MSG_WINLEN	; [4]
		blo 1f			; [3]
		ldy #scroll_8	; [4]
		jmp loop		; [3] *19*
1		ldy #seq_pl5	; [4]
		jmp loop		; [3] *19*

; reset message to start
scroll_8
		mac_delay_cycles 8
		ldx #MSG_TEXT	; [5]
		stx msgstart	; [5]
		ldy #seq_pl5	; [4]
		jmp loop		; [3] *17*
		
; set up note duration counter
seq_pl5
		mac_delay_cycles 10
		
		ldx #550		; [3]
		stx ntcnt		; [5]

		ldy #seq_pl6	; [4]
		jmp loop		; [3] *15*

; sequencer stays here for duration of note
seq_pl6
		ldx ntcnt		; [5]
		leax -1,x		; [5]
		stx ntcnt		; [5]
		bne 1f			; [3]

		ldy #seq_st		; [4]
		jmp loop		; [3] *25*
		
1		mac_delay_cycles 4
		jmp loop		; [3] *21*

	
;--------------------------------------------------------------
; LOADER CODE
;
; Each wave from the tape is generally handled by 3 routines
;  xxxxA - Reset timer & do first sample
;  xxxxB - Continue sampling first half of wave
;  xxxxC - Sample last half of wave and determine bit
;
; Routines are padded to 33 cycles
; (Or 58 cycles if taking sequencer slot)
;--------------------------------------------------------------

; set up for sync byte
sync_st
		mac_delay_cycles 4

		ldu #blktyp		; [3] load 2 header bytes directly into variables
		ldd #2			; [3]
		stb bytcnt		; [4]
		sta ,u			; [4]
		sta cksum		; [4]
		sta hdflag		; [4]
		lds #syncA		; [4]
		jmp ,y			; [3] *29*

; do first sample of wave
; check if sync acquired yet
syncA
		clr timer		; [6]
		lda pia1		; [5]
		lsra			; [2]
bcs1	bcs 1f			; [3]

		lda <<0,u		; [5]
		cmpa #$3c		; [2]
		bne 2f			; [3]

		lds #read1C		; [4]
		jmp ,y			; [3] *33*

2		lds #syncC		; [4]
		jmp ,y			; [3] *33*

1		mac_delay_cycles 10
		lds #syncB		; [4]
		jmp ,y			; [3] *23*

; continue sampling first half of wave
; check for sync on alternate path
syncB
		inc timer		; [6]
		lda pia1		; [5]
		lsra			; [2]
bcs2	bcs 1f			; [3]

		lda <<0,u		; [5]
		cmpa #$3c		; [2]
		bne 2f			; [3]

		lds #read1C		; [4]
		jmp ,y			; [3] *33*

2		lds #syncC
		jmp ,y			; [3] *33*

1		mac_delay_cycles 14
		jmp ,y			; [3] *19*

; sample last half of wave
; store bit & repeat
syncC
		ldb timer		; [4]
		incb			; [2]
		lda pia1		; [5]
		mac_delay_cycles 2
		lsra			; [2]
bcc1	bcc 1f			; [3]

		cmpb #MIDVAL	; [2]
		ror ,u			; [6]
		
		lds #syncA		; [4]
		jmp ,y			; [3] *31*

1		stb timer		; [4]
		mac_delay_cycles 8
		jmp ,y			; [3] *23*

;---------------------------------------
; code jumps here after acquiring sync
; we've already started reading first bit so need to complete it

; sample last half of wave then store bit
read1C
		ldb timer		; [4]
		incb			; [2]
		lda pia1		; [5]
		mac_delay_cycles 2
		lsra			; [2]
bcc2	bcc 1f			; [3]

		cmpb #MIDVAL	; [2]
		ror ,u			; [6]

		lds #read2A		; [4]
		jmp ,y			; [3] *31*

1		stb timer		; [4]
		mac_delay_cycles 8
		jmp ,y			; [3] *23*

; do first sample of wave (2nd bit)
; set up bit loop counter
read2A
		clr timer		; [6]
		lda #6			; [2]
		sta bitcnt		; [4]
		lda pia1		; [5]
		mac_delay_cycles 4
		lsra			; [2]
bcs3	bcs 1f			; [3]

		lds #read2C		; [4]
		jmp ,y			; [3] *29*

1		lds #read2B		; [4]
		jmp ,y			; [3] *29*

; continue sampling first half of wave (2nd bit)
read2B
		inc timer		; [6]
		lda pia1		; [5]
		mac_delay_cycles 10
		lsra			; [2]
bcs4	bcs 1f			; [3]

		lds #read2C		; [4]
		jmp ,y			; [3] *23*

1		mac_delay_cycles 4
		jmp ,y			; [3] *23*

; sample last half of wave then store bit (2nd bit)
read2C
		ldb timer		; [4]
		incb			; [2]
		lda pia1		; [5]
		mac_delay_cycles 2
		lsra			; [2]
bcc3	bcc 1f			; [3]

		cmpb #MIDVAL	; [2]
		ror ,u			; [6]

		lds #read3A		; [4]
		jmp ,y			; [3] *31*

1		stb timer		; [4]
		mac_delay_cycles 8
		jmp ,y			; [3] *23*


; do first sample of wave (bits 3-8)
read3A
		clr timer		; [6]
		lda pia1		; [5]
		lsra			; [2]
bcs5	bcs 1f			; [3]

		dec bitcnt		; [6]
		bne 2f			; [3]

		lds #read4C		; [4]
		jmp <<0,y		; [4] *33*

2		lds #read3C		; [4]
		jmp <<0,y		; [4] *33*

1		lds #read3B		; [4]
		mac_delay_cycles 10
		jmp ,y			; [3] *23*


; continue sampling first half of wave (bits 3-8)
read3B
		inc timer		; [6]
		lda pia1		; [5]
		lsra			; [2]
bcs6	bcs 1f			; [3]

		dec bitcnt		; [6]
		bne 2f			; [3]

		lds #read4C		; [4]
		jmp <<0,y		; [4] *33*

2		lds #read3C		; [4]
		jmp <<0,y		; [4] *33*

1		mac_delay_cycles 14
		jmp ,y			; [3] *19*


; sample last half of wave then store bit (bits 3-7)
; loop back to get next bit
read3C
		ldb timer		; [4]
		incb			; [2]
		lda pia1		; [5]
		mac_delay_cycles 2
		lsra			; [2]
bcc4	bcc 1f			; [3]

		cmpb #MIDVAL	; [2]
		ror ,u			; [6]

		lds #read3A		; [4]
		jmp ,y			; [3] *31*

1		stb timer		; [4]
		mac_delay_cycles 8
		jmp ,y			; [3] *23*

; sample last half of wave then store bit (bit 8)
; update checksum and check for end of header or block
read4C
		ldb timer		; [4]
		incb			; [2]
		lda pia1		; [5]
		mac_delay_cycles 6
		lsra			; [2]
bcc5	bcc 1f			; [3]

		cmpb #MIDVAL	; [2]
		ror ,u			; [6]

		lda ,u			; [4] add new byte to checksum
		adda cksum		; [4]
		sta cksum		; [4]

		dec bytcnt		; [6]
		bne 2f			; [3] not end of block

		lds #blck1A		; [4]
		jmp loop		; [3] *52* no time for sequencer
		
2		lds #read1A		; [4]
		jmp loop		; [3] *52* no time for sequencer

1		stb timer		; [4]
		mac_delay_cycles 4
		jmp ,y			; [3] *23*


; do first sample of wave (1st bit of next byte)
; advance data pointer
read1A
		leau 1,u		; [5]
		clr timer		; [6]
		lda pia1		; [5]
		mac_delay_cycles 5
		lsra			; [2]
bcs7	bcs 1f			; [3]

		lds #read1C		; [4]
		jmp ,y			; [3] *28*

1		lds #read1B		; [4]
		jmp ,y			; [3] *28*

; continue sampling first half of wave
; loop back to complete 1st bit
read1B
		inc timer		; [6]
		lda pia1		; [5]
		mac_delay_cycles 10
		lsra			; [2]
bcs8	bcs 1f			; [3]

2		lds #read1C		; [4]
		jmp ,y			; [3] *23*

1		mac_delay_cycles 4
		jmp ,y			; [3] *19*

;-----------------------------------------

blck1A
		com hdflag		; [6]
		bne 2f			; [3] done block header, now get data

		lda cksum		; [4] check block loaded ok
		suba ,u			; [4] last byte loaded was cksum, so sub it
		cmpa ,u			; [4]
		beq 1f			; [3] (24) cksum OK

		jmp ioerror

1		lda blktyp		; [4]
		beq 5f			; [3] (31) filename block
		bpl 6f			; [3] (34) not eof block
		
		jmp exit


5		ldx FILE_HEADER+13	; [6] file load address
		stx ldaddr			; [6]
		ldx FILE_HEADER+11	; [6] file exec address
		stx execad			; [5]
		
		lds #sync_st	; [4] wait for next block
		jmp loop		; [3] *61* no time for sequencer
		
6		ldx ldaddr		; [5] address to load next block
		ldb blklen		; [4]
		abx				; [3]
		stx ldaddr		; [5]

		lds #sync_st	; [4] wait for next block
		jmp loop		; [3] *58* no time for sequencer
		
2		lda $400		; [5] blink
		eora #8			; [2]
		sta $400		; [5]
		ldu ldaddr		; [5] file load address
		lda blklen		; [4] get number of data bytes
		inca			; [2] add one to load checksum as well
		sta bytcnt		; [4] (works with any value of block length 0-255)
		
		clr timer		; [6]
		lda pia1		; [5]
		lsra			; [2]
bcs9	bcs 1f			; [3]

		lds #read1C		; [4] wait for last half of wave
		jmp loop		; [3] *59* no time for sequencer

1		lds #read1B		; [4] continue sampling 1st half of wave
		jmp loop		; [3] *59* no time for sequencer


;-----------------------------------------
; anti-phase fixup tables
bcctab	fdb bcc1,bcc2,bcc3,bcc4,bcc5,0
bcstab	fdb bcs1,bcs2,bcs3,bcs4,bcs5,bcs6,bcs7,bcs8,bcs9,0

;-----------------------------------------
; setup code to run before loader starts

setup
;		ldd #$ffff			; clear screen
;		ldx #$400
;1		std ,x++
;		cmpx #$400+$1e00
;		blo 1b
		
;		lda #$fc		; set up graphics mode
;		sta $ff22		;
;		sta $ffc5		;
;		sta $ffc3		;

		lda phase
		beq phaseok
		
		; fixup for anti-phase
		ldd #$2425		; bcc bcs
		ldx #bcctab
1		ldu ,x++
		beq 2f
		stb ,u
		bra 1b
2		ldu ,x++
		beq phaseok
		sta ,u
		bra 2b
		
phaseok		
		lda $ff23		; sound on
		ora #$8			;
		sta $ff23		;
		
		lda	pia1+1		; motor on
		ora	#8			;
		sta	pia1+1		;

		; load start address
		ldu #FILE_HEADER
		stu ldaddr

		clr blnkflg
		
		; initialise sequencer
		clra
		clrb
		std freq1+1
		std freq2+1
		ldd #NOPAT
		std curnote
		ldd #PATTERNS-2
		std curpat

		; initialise scroller
		ldd #MSG_TEXT
		std msgstart
		
		rts

;-----------------------------------------
; display i/o error message

errmsg	fcb /IO/,$60,/ERROR/,$60,':+$40,'(+$40,0

ioerror
		ldx #$400	; display io error message
		ldu #errmsg
1		lda ,u+
		beq 2f
		sta ,x+
		bra 1b

2		lda #$60	; clear rest of screen
3		sta ,x+
		cmpx #$600
		blo 3b

		;lda #4			; restore text mode
		;sta $ff22
		;sta $ffc4
		;sta $ffc2
		;sta $ffc0

		lds tempstk+2	; restore stack
		bsr motoroff
		puls cc,dp,pc
		
; code to run before loader exit
exit
tempstk	lds #0			; restore stack
		bsr motoroff
		puls cc,dp
		jmp [execad]
		
motoroff
		lda	pia1+1		; motor off
		anda #$f7		;
		sta	pia1+1		;
1		rts
		
;-----------------------------------------

;equal tempered 12 note per octave frequency table for 9.1KHz

freqtab
	fdb 0,471,499,529,560,593,629,666,706,748,792,839
	fdb 889,942,998,1057,1120,1187,1258,1332,1412,1495,1584,1679		; B2 -
	fdb 1778,1884,1996,2115,2241,2374,2515,2665,2823,2991,3169,3357		; B3 -
	fdb 3557,3768,3992,4230,4481,4748,5030,5329,5646,5982,6338,6714		; B4 -
	fdb 7114,7537,7985,8460,8963,9496,10060,10658,11292,11964,12675
	fdb 13429,14227,15073,15970
	
;-----------------------------------------

	include "music.asm"
	include "message.asm"
	include "textscreen.asm"
	
	end start
